statuses   = require("statuses")
isType     = require("@rill/is-type")
byteLength = require("./byteLength")
browser    = require("./browser") if process.browser
noop       = Promise.resolve()
htmlReg    = /^\s*</

###
# A default middleware that handles sending the response for the request.
#
# @param {Object} req
# @param {Object} res
# @param {Function} next
###
module.exports = (req, res, next)->
	# Skip request if something had already ended it.
	return noop if res.headersSent

	{ method, redirectURL }                                 = req
	{ res: originalRes, headers, status, statusText, body } = res

	# Handle cookies isomorphically.
	if process.browser
		cookies = req.cookies.serialize(true)
		document.cookie = cookie for cookie in cookies if cookies
	else
		cookies = req.cookies.serialize()
		headers["set-cookie"] = cookies if cookies

	# Redirects isomorphically.
	if redirectURL?
		headers["location"] ?= redirectURL
		status = 302 unless statuses.redirect[status]

		if process.browser
			browser.goTo(redirectURL)
			return noop

	# Default the status to 200 if there is substance to the response.
	status      = 200 if Number(status) is 404 and body?
	statusText ?= statuses[status]

	if statuses.empty[status] or not body?
		delete headers["content-type"]
		body = undefined

	else if isType.String(body)
		headers["content-type"] ?= "text/#{if htmlReg.test(body) then "html" else "plain"}; charset=UTF-8"

	else if isType.Buffer(body)
		headers["content-length"] = body.length

	else if isType.Stream(body)
		headers["content-type"] ?= "application/octet-stream"

	else try
		body                    = JSON.stringify(body)
		headers["content-type"] ?= "application/json; charset=UTF-8"

	unless process.browser
		if "HEAD" is method
			originalRes.writeHead(status, statusText, headers)
			originalRes.end()
		else if isType.Stream(body)
			originalRes.writeHead(status, statusText, headers)
			body.pipe(originalRes)
		else
			headers["content-length"] ?= byteLength(body)
			originalRes.writeHead(status, statusText, headers)
			originalRes.end(body)

	return noop

